#!/usr/bin/env python
# -*- coding: utf-8 -*-

import string
import sys
from ConfigParser import *

class Config:
    def __init__ (self, cfgFile):
        "Reads the config file and returns a valid object if parse is successful"
        self.cfgFilename = cfgFile
        try:
            file = open(self.cfgFilename, "r")
            file.close()
            self.config = ConfigParser()
            self.config.read(self.cfgFilename)
        except ParsingError:
            self.config = None
        except IOError:
            self.config = None
            
    def isValid(self):
        "Returns non-zero if the configuration is valid"
        return (self.config is not None)

    def getValue(self, section, option, default):
        try:
            return self.config.get(section, option)
        except:
            return default
                
    def getFileSuffix(self):
        return self.getValue("Extensions", "File", "po")

    def getTemplateSuffix(self):
        return self.getValue("Extensions", "Template", "pot")

    def getRepository(self):
        return self.getValue("Directories","Repository", None)

    def getFilesRoot(self):
        return self.getValue("Directories","Messages", "pt/messages")

    def getTemplatesRoot(self):
        return self.getValue("Directories","MessagesTemplates",
                             "templates")

    def getDocsRoot(self):
        return self.getValue("Directories","Docs",
                             self.getFilesRoot() + "/docs")

    def getTemplateDocsRoot(self):
        return self.getValue("Directories","DocsTemplates",
                             self.getTemplatesRoot() + "/docs")

    def hasDirDetection(self):
        status = self.getValue("Modules","AutoDetection", "1")
        return int(status)


    def getModuleExceptions(self):
        status = self.getValue("Modules","Exceptions", "CVS docs")
        return filter((lambda x: x != ""), string.split(status, ' '))
    
    def getDirectories(self):
        status = self.getValue("Modules","Directories", "")
        return filter((lambda x: x != ""), string.split(status, ' '))

    def getMappings(self):
        status = self.getValue("Modules","Mappings", "")
        mapStringsList = filter((lambda x: x != ""), string.split(status, ' '))
        mappings = {}
        for mapping in mapStringsList:
            (virtual, real) = string.split(mapping,"=")
            mappings[virtual] = real
        return mappings

    def getMapping(self, module):
        mappings = self.getMappings()
        try:
            return mappings[module]
        except KeyError:
            return module

    def toUTF8Command(self, filename = None):
        cmd = self.getValue("Commands","ToUTF8", "record latin1..utf8 %1")
        if (filename is not None):
            cmd = string.replace(cmd, "%1", filename)
        return cmd
        
    def toISO8859Command(self, filename = None):
        cmd = self.getValue("Commands","ToLatin1", "record utf8..latin1 %1")
        if (filename is not None):
            cmd = string.replace(cmd, "%1", filename)
        return cmd

    def getStatsCommand(self, filename = None):
        cmd = self.getValue("Commands","Statistics",
                            "msgfmt --statistics -o /dev/null %1 2>&1")
        if (filename is not None):
            cmd = string.replace(cmd, "%1", filename)
        return cmd

    def getIgnores(self, module):
        return []
    
if __name__ == '__main__':
    if (len(sys.argv) > 1):
        filename = sys.argv[1]
        conf = Config(filename)
        if conf.isValid():
            print "CONFIGURATION FOR FILE '%s':\n\n" % filename
            print "Extension for regular files: %s" % conf.getFileSuffix()
            print "Extension for template files: %s" % conf.getTemplateSuffix()
            print "Repository working directory: %s" % conf.getRepository()
            print "Sub-directory for files: %s" % conf.getFilesRoot()
            print "Sub-directory for templates: %s" % conf.getTemplatesRoot()
            print "Sub-directory for doc files: %s" % conf.getDocsRoot()
            print "Sub-directory for doc template files: %s" % conf.getTemplateDocsRoot()
            print ""
            print "Has directory auto-detection: %d" % conf.hasDirDetection()
            if (conf.hasDirDetection()):
                print "Directory Exceptions: %s" % str(conf.getModuleExceptions())
            else:
                print "Directories: %s" % str(conf.getDirectories())
                
            print ""
            print "Mappings for modules: " + str(conf.getMappings())
            sys.exit(0)
        else:
            print "File %s doesn\'t exist or has an invalid format!" % filename
            sys.exit(1)
    else:
        print "Usage: Config.py <config file>"
        sys.exit(1)
