#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import glob

from Config import *
from File import *
from Status import *

class Module(Status):
    def __init__(self, config, root, fileDir, templateDir, moduleName):
        Status.__init__(self, config)
        self.config = config
        self.repositoryRoot = root
        self.moduleName = moduleName
        self.realName = config.getMapping(moduleName)
        self.fileObjects = []
        
        # Modules may, in fact, not exist inside directories
        self.fileUrl = os.path.join(fileDir, self.realName)
        self.templateUrl = os.path.join(templateDir, self.realName)
        self.fileDir = os.path.join(root, self.fileUrl)
        self.templateDir = os.path.join(root, self.templateUrl)
        
    def getName(self):
        return self.moduleName

    def getRealName(self):
        return self.realName
    
    def isValid(self):
        return (os.path.isdir(self.fileDir) or os.path.isdir(self.templateDir))

    def isNew(self):
        return (os.path.isdir(self.templateDir) and
                not (os.path.isdir(self.fileDir)))

    def isObsolete(self):
        return (os.path.isdir(self.fileDir) and
                not (os.path.isdir(self.templateDir)))

    def getTemplateUrl(self):
        return self.templateUrl

    def getFileUrl(self):
        return self.fileUrl

    def getTemplateDir(self):
        return self.templateDir

    def getFileDir(self):
        return self.fileDir
    
    def getFiles(self, dir, suffix):
        files = []
        filePattern = os.path.join(dir,"*." + suffix)
        tempFiles = glob.glob(filePattern)
        tempFiles = filter ((lambda f: not os.path.isdir(f)), tempFiles)
        for file in tempFiles:
            file = os.path.basename(file)
            file = string.replace(file, "." + suffix, "")
            files.append(file)
            
        return files

    def getTotalFiles(self, files, templates):
        newFiles = filter((lambda f: f not in templates), files)
        templates.extend(newFiles)
        return templates

    def toXML(self, level = 0):
        status = self.run()
        if (status):
            attributes, content = "", ""
            subContent = self.textTag("name", self.getName(), level + 1,
                                      self.attribute("real",
                                                     self.getRealName()))

            if self.isValid():
                # The module is valid. If so, set state to VALID and show
                # its contents
                state = "VALID"
            elif self.isNew():
                state = "NEW"
            elif self.isObsolete():
                state = "OBSOLETE"
            else:
                state = "INVALID"

            # Generate the XML content for each file inside the module
            subContent = subContent + self.openTag("files", level + 1)

            for file in self.fileObjects:
                subContent = subContent + file.toXML(level + 2)
                
            subContent = subContent + self.closeTag("files", level + 1)
            
            # Define the module status as attribute for the 'module' object
            if (attributes != ""):
                attributes = attributes + self.attribute("state", state)
            else:
                attributes = self.attribute("state", state)

            content = self.openTag("module", level, attributes) + \
                      subContent + \
                      self.closeTag("module", level)
            return content
        else:
            return ""
            
    
    def run(self):
        sys.stderr.write("MODULE INFO:" + "\n")
        sys.stderr.write("Name:        [%s]" % self.getName() + "\n")
        sys.stderr.write("Mapping:     [%s]" % self.getRealName() + "\n")
        sys.stderr.write("Is valid:    [%s]" % self.isValid() + "\n")
        if self.isValid():
            sys.stderr.write("Is new:     [%s]" % self.isNew() + "\n")
            sys.stderr.write("Is obsolete:[%s]" % self.isObsolete() + "\n")
            if (self.isValid()):
                filesList = self.getFiles(self.getFileDir(),
                                          self.config.getFileSuffix())
                
                templatesList = self.getFiles(self.getTemplateDir(),
                                              self.config.getTemplateSuffix())
            
                totalFiles = self.getTotalFiles(filesList, templatesList)
                self.fileObjects = []
                for file in totalFiles:
                    sys.stderr.write("Adding file '%s'" % file + "\n")
                    fileObject = File(self, file)
                    if (fileObject.run()):
                        self.fileObjects.append(fileObject)
                    else:
                        return 0
                        
        # Even if the files below return invalid states, return true always
        return 1

