#!/usr/bin/env python

import os
import sys

from Config import *
from Status import *
from Module import *

class POStatus(Status):

    def __init__(self, config, rootTag = "", fileDir = None, templateDir = None):
        Status.__init__(self,config)
        self.rootTag = rootTag
        self.fileDir = fileDir
        self.templateDir = templateDir
        self.modules = []

    def toXML(self, level = 0):
        content = self.openTag(self.rootTag, level)
        status = self.run()
        if (status):
            content = content + self.openTag("modules", level + 1)
            for moduleName in self.modules:
                content = content + moduleName.toXML(level + 2)
            content = content + self.closeTag("modules", level + 1)
        else:
            return None
        
        content = content + self.closeTag(self.rootTag, level)
        return content

    def getDirectories(self, templateDir):
        "This method returns a list of the directories inside the templates"
        cfg = self.getConfig()
        exceptions = cfg.getModuleExceptions()
        
        if (cfg.hasDirDetection()):
            # Directories are extracted automatically, if detection is enabled
            dirs = os.listdir(templateDir)
            dirs = filter((lambda m:
                           os.path.isdir(os.path.join(templateDir, m))),
                          dirs)
        else:
            # No auto-detection <=> get the list from the config file
            dirs = cfg.getDirectories()

        # Delete the module exceptions (directories to ignore)
        dirs = filter((lambda d: d not in exceptions), dirs)
        return dirs

    def run(self):
        cfg = self.getConfig()
        fileDir = os.path.join(cfg.getRepository(), self.fileDir)
        templateDir = os.path.join(cfg.getRepository(), self.templateDir)
        
        if (fileDir is not None and templateDir is not None):
            # Let's assume a valid string, but validate if both are directories
            if (os.path.isdir(fileDir) and os.path.isdir(templateDir)):

                # Try to find both modules in templates and actual files
                # Then, generate a new list which is the union of both modules
                templateModules = self.getDirectories(templateDir)
                poModules = self.getDirectories(fileDir)
                
                i18nModules = filter((lambda m: m not in templateModules),
                                     poModules)
                i18nModules.extend(templateModules)

                sys.stderr.write("TEMPLATE MODULES: <%s>\n" % templateModules)
                sys.stderr.write("FILE MODULES: <%s>\n" % poModules)
                sys.stderr.write("MODULES UNION: <%s>\n" % i18nModules)
                for moduleName in i18nModules:
                    sys.stderr.write("Module: %s\n" % moduleName)
                    module = Module(cfg, cfg.getRepository(),
                                    self.fileDir, self.templateDir, moduleName)
                    self.modules.append(module)
                    
                return 1
            else:
                return 0
        else:
            return 0
        
        return 1

if __name__ == '__main__':
    if len(sys.argv) > 1:
        filename = sys.argv[1]
        cfg = Config(filename)
        if not cfg.isValid():
            sys.stderr.write('Error parsing config file\n')
            sys.exit(1)

        print "<?xml version='1.0' encoding='iso-8859-15'?>\n"
        print "<po-status>"
        
        appStatus = POStatus(cfg, "apps",
                             cfg.getFilesRoot(), cfg.getTemplatesRoot())

        content = appStatus.toXML(level = 1)
        if (content is not None):
            print content
                
            sys.stderr.write("Status for apps successful\n")
        else:
            sys.stderr.write("Status for apps UNSUCCESSFUL")
            
        docStatus = POStatus(cfg, "docs",
                             cfg.getDocsRoot(), cfg.getTemplateDocsRoot())

        content = docStatus.toXML(level = 1)
        if (content is not None):
            print content
            sys.stderr.write("Status for docs successful\n")
        else:
            sys.stderr.write("Status for docs UNSUCCESSFUL\n")
            
        print "</po-status>"
        sys.exit(0)
    else:
        sys.stderr.write("Usage: %s <config file>\n" % sys.argv[0])
        sys.exit(1)
